package edu.ucla.loni.debabel.processors.keith.calculus;

import edu.ucla.loni.flow.InputNode;
import edu.ucla.loni.flow.OutputNode;
import edu.ucla.loni.flow.exceptions.ProcessingException;
import edu.ucla.loni.flow.process.Processor;
import edu.ucla.loni.flow.process.ProcessorProperty;

/**
 * Processor that computes the cross product of two vectors.
 *
 * @author Keith R. Ouellette
 *
 * @version 26 June 2003
 */
public class CrossProduct extends Processor
{
  /** Description of this Processor. */
  private static String DESC = "Computes the cross product of two vectors.";

  /** Input Node Name for the first vector's x coordinate. */
  private static String COORD_X1_NAME = "First Vector's X Coordinate";

  /** Input Node description for the first vector's x coordinate. */
  private static String COORD_X1_DESC = "x1"; 

  /** Input Node Name for the first vector's y coordinate. */
  private static String COORD_Y1_NAME = "First Vector's Y Coordinate";

  /** Input Node description for the first vector's y coordinate. */
  private static String COORD_Y1_DESC = "y1";  

  /** Input Node Name for the first vector's z coordinate. */
  private static String COORD_Z1_NAME = "First Vector's Z Coordinate";

  /** Input Node description for the first vector's z coordinate. */
  private static String COORD_Z1_DESC = "z1"; 
  
 /** Input Node Name for the second vector's x coordinate. */
  private static String COORD_X2_NAME = "Second Vector's X Coordinate";

  /** Input Node description for the second vector's x coordinate. */
  private static String COORD_X2_DESC = "x2"; 

  /** Input Node Name for the second vector's y coordinate. */
  private static String COORD_Y2_NAME = "Second Vector's Y Coordinate";

  /** Input Node description for the second vector's y coordinate. */
  private static String COORD_Y2_DESC = "y2";  

  /** Input Node Name for the second vector's z coordinate. */
  private static String COORD_Z2_NAME = "Second Vector's Z Coordinate";

  /** Input Node description for the second vector's z coordinate. */
  private static String COORD_Z2_DESC = "z2"; 

  /** Output Node Name for the cross product of the vectors. */
  private static String COORD_CROSS1_NAME = "Cross Product X Coordinate";

  /** Output Node description for the cross product. */
  private static String COORD_CROSS1_DESC = "X Coordinate of v1 x v2";

  /** Output Node Name for the cross product of the vectors. */
  private static String COORD_CROSS2_NAME = "Cross Product Y Coordinate";

  /** Output Node description for the cross product. */
  private static String COORD_CROSS2_DESC = "Y Coordinate of v1 x v2";

  /** Output Node Name for the cross product of the vectors. */
  private static String COORD_CROSS3_NAME = "Cross Product Z Coordinate";

  /** Output Node description for the cross product. */
  private static String COORD_CROSS3_DESC = "Z Coordinate of v1 x v2";

  /**
   * Constructs a CrossProduct with default Properties.
   *
   * @param name Name of the Processor.
   *
   * @return IllegalStringException If the name contains illegal characters.
   */
  public CrossProduct(String name)
    {
      super(name, DESC);

      // Create the InputNodes for the two vectors
      Class type = Double.class;
      InputNode inputNode = new InputNode(COORD_X1_NAME, COORD_X1_DESC, type);
      _addInputNode(inputNode);

      type = Double.class;
      inputNode = new InputNode(COORD_Y1_NAME, COORD_Y1_DESC, type);
      _addInputNode(inputNode);

      type = Double.class;
      inputNode = new InputNode(COORD_Z1_NAME, COORD_Z1_DESC, type);
      _addInputNode(inputNode);

      type = Double.class;
      inputNode = new InputNode(COORD_X2_NAME, COORD_X2_DESC, type);
      _addInputNode(inputNode);

      type = Double.class;
      inputNode = new InputNode(COORD_Y2_NAME, COORD_Y2_DESC, type);
      _addInputNode(inputNode);

      type = Double.class;
      inputNode = new InputNode(COORD_Z2_NAME, COORD_Z2_DESC, type);
      _addInputNode(inputNode);

      // Create the OutputNodes for the cross product
      type = Double.class;
      OutputNode node = new OutputNode(COORD_CROSS1_NAME, COORD_CROSS1_DESC, 
				       type);
      _addOutputNode(node);

      type = Double.class;
      node = new OutputNode(COORD_CROSS2_NAME, COORD_CROSS2_DESC, type);
      _addOutputNode(node);

      type = Double.class;
      node = new OutputNode(COORD_CROSS3_NAME, COORD_CROSS3_DESC, type);
      _addOutputNode(node);
    }

  /**
   * Constructs a CrossProduct using the specified Properties.
   *
   * @param name Name of the Processor.
   * @param properties Properties for the Processor.
   *
   * @return IllegalStringException If the name contains illegal characters.
   */
  public CrossProduct(String name, ProcessorProperty[] properties)
    {
      this(name);
    }

  /**
   * Generates values for the Output Nodes using the current values of the
   * Input Nodes.
   *
   * @throws ProcessingException If an error occurs during processing.
   */
  public void generateOutputValues() throws ProcessingException
    {
      try {
	// Get the values of the coordinates of the two vectors
	Double vx1 = (Double)_getInputValue(COORD_X1_NAME, null);
	Double vy1 = (Double)_getInputValue(COORD_Y1_NAME, null);
	Double vz1 = (Double)_getInputValue(COORD_Z1_NAME, null);
	Double vx2 = (Double)_getInputValue(COORD_X2_NAME, null);
	Double vy2 = (Double)_getInputValue(COORD_Y2_NAME, null);
	Double vz2 = (Double)_getInputValue(COORD_Z2_NAME, null);
	
	double x1 = vx1.doubleValue();
	double y1 = vy1.doubleValue();
	double z1 = vz1.doubleValue();
	double x2 = vx2.doubleValue();
	double y2 = vy2.doubleValue();
	double z2 = vz2.doubleValue();
	
	// Compute the cross product (assume a right-handed coordinate system
	Double cx1 = new Double(y1*z2 - z1*y2);
	Double cx2 = new Double(z1*x2 - x1*z2);
	Double cx3 = new Double(x1*y2 - y1*x2);

	// Set the cross product
	_setOutputValue(COORD_CROSS1_NAME, cx1);
	_setOutputValue(COORD_CROSS2_NAME, cx2);
	_setOutputValue(COORD_CROSS3_NAME, cx3);
      }
      catch (Exception e) {
	String msg = "Unable to generate output values for \"" + this + "\".";
	throw new ProcessingException(msg, e);
      }
    }
}
