/*
  COPYRIGHT NOTICE
  Copyright (c) 2009  Scott C. Neu and Arthur W. Toga

  See README.license for license notices.
*/

package edu.ucla.loni.debabel.processors.scott;

import edu.ucla.loni.flow.InputNode;
import edu.ucla.loni.flow.OutputNode;
import edu.ucla.loni.flow.exceptions.ProcessingException;
import edu.ucla.loni.flow.process.Processor;
import edu.ucla.loni.flow.process.ProcessorProperty;
import java.io.File;

/**
 * Processor that determines the size of a file.
 *
 * @version 20 January 2009
 */
public class FileSizer extends Processor
{
    /** Description of this Processor. */
    private static String DESC = "Determines the size of a file.";

    /** Input Node name for the file name. */
    private static String FILE_NAME = "File Name";

    /** Input Node description for the file name. */
    private static String FILE_DESC = "Name of the file.";

    /** Output Node name for the file size. */
    private static String SIZE_NAME = "File Size";

    /** Output Node description for the file size. */
    private static String SIZE_DESC = "Size of the file (in bytes)";

    /**
     * Constructs a File Sizer.
     *
     * @param name Name of the Processor.
     *
     * @return IllegalStringException If the name contains illegal characters.
     */
    public FileSizer(String name)
    {
	super(name, DESC);

	// Create the Input Node for the file name
	Class type = String.class;
	InputNode inputNode = new InputNode(FILE_NAME, FILE_DESC, type);
	_addInputNode(inputNode);

	// Create the Output Node for the truncated file name
	type = Long.class;
	OutputNode outputNode = new OutputNode(SIZE_NAME, SIZE_DESC, type);
	_addOutputNode(outputNode);
    }

    /**
     * Constructs a File Sizer using the specified Properties.
     *
     * @param name Name of the Processor.
     * @param properties Properties for the Processor.
     *
     * @return IllegalStringException If the name contains illegal characters.
     */
    public FileSizer(String name, ProcessorProperty[] properties)
    {
	this(name);
    }

    /**
     * Generates values for the Output Nodes using the current values of the
     * Input Nodes.
     *
     * @throws ProcessingException If an error occurs during processing.
     */
    public void generateOutputValues() throws ProcessingException
    {
	long fileSize = 0;

	try {

	    // Get the file name
	    String fileName = (String)_getInputValue(FILE_NAME, null);
	    if (fileName != null) {
		File file = new File(fileName);

		// Set the file size
		fileSize = file.length();
	    }
	}

	// File I/O error
	catch (Exception e) {}

	// Set the output value
	_setOutputValue(SIZE_NAME, new Long(fileSize));
    }
}
