/*
COPYRIGHT NOTICE
Copyright (c) 2006  Scott C. Neu and Arthur W. Toga

See README.license for license notices.
 */

package edu.ucla.loni.debabel.processors.scott;

import edu.ucla.loni.flow.InputNode;
import edu.ucla.loni.flow.OutputNode;
import edu.ucla.loni.flow.exceptions.ProcessingException;
import edu.ucla.loni.flow.process.Processor;
import edu.ucla.loni.flow.process.ProcessorProperty;

/**
 * Processor that trims spaces before and after a string.
 *
 * @version 25 May 2006
 */
public class StringTrimmer extends Processor
{
    /** Description of this Processor. */
    private static String DESC = "Removes spaces before and after a string.";

    /** Input Node name for the String to trim. */
    private static String STRING_NAME = "String";

    /** Input Node description for the String to truncate. */
    private static String STRING_DESC = "String to trim.";
    
    /** Output Node name for the trimmed String. */
    private static String TRIMMED_NAME = "Trimmed String";

    /** Output Node description for the trimmed String. */
    private static String TRIMMED_DESC = "String whose end is trimmed.";

    /**
     * Constructs a String Trimmer with default Properties.
     *
     * @param name Name of the Processor.
     *
     * @return IllegalStringException If the name contains illegal characters.
     */
    public StringTrimmer(String name)
    {
	super(name, DESC);

	// Create the Input Node
	Class type = String.class;
	InputNode inputNode = new InputNode(STRING_NAME, STRING_DESC, type);
	_addInputNode(inputNode);

	// Create the Output Node
	type = String.class;
	OutputNode node = new OutputNode(TRIMMED_NAME, TRIMMED_DESC, type);
	_addOutputNode(node);
    }

    /**
     * Constructs a String Trimmer using the specified Properties.
     *
     * @param name Name of the Processor.
     * @param properties Properties for the Processor.
     *
     * @return IllegalStringException If the name contains illegal characters.
     */
    public StringTrimmer(String name, ProcessorProperty[] properties)
    {
	this(name);
    }

    /**
     * Generates values for the Output Nodes using the current values of the
     * Input Nodes.
     *
     * @throws ProcessingException If an error occurs during processing.
     */
    public void generateOutputValues() throws ProcessingException
    {
	try {

	    // Get the input value
	    String s = (String)_getInputValue(STRING_NAME, "");

	    // Trim the value
	    s = s.trim();

	    // Set the output value
	    _setOutputValue(TRIMMED_NAME, s);
	}

	catch (Exception e) {
	    String msg = "Unable to generate output values for \"" + this +
		"\".";
	    throw new ProcessingException(msg, e);
	}
    }
}
