/*
  COPYRIGHT NOTICE
  Copyright (c) 2011  Scott C. Neu and Arthur W. Toga

  See README.license for license notices.
*/

package edu.ucla.loni.debabel.processors.scott;

import edu.ucla.loni.dataflow.MetadataTree;
import edu.ucla.loni.dataflow.identify.GeneralMetadataTree;
import edu.ucla.loni.dataflow.identify.GeneralMetadataValueTree;
import edu.ucla.loni.dataflow.translate.SingleSourceDataTree;
import edu.ucla.loni.dataflow.translate.image.BufferedImageAccessor;
import edu.ucla.loni.dataflow.translate.image.ImageTree;
import edu.ucla.loni.flow.Restriction;
import edu.ucla.loni.flow.restrictions.RangeRestriction;
import edu.ucla.loni.flow.restrictions.SetRestriction;
import java.awt.image.BufferedImage;
import java.awt.image.DataBuffer;
import java.util.Vector;

/**
 * Utility methods for Scott's Processors.
 *
 * @version 14 February 2011
 */
public class Utilities
{
    /**
     * Gets the Restriction for Classes that occur in Scott's Processors.
     *
     * @return Set Restriction for Classes that occur in Scott's Processors.
     */
    public static Restriction getClassRestriction()
    {
	Vector optionSet = new Vector();

	// Classes that can occur in Scott's Processors
	optionSet.add( Boolean.class );
	optionSet.add( Boolean[].class );
	optionSet.add( BufferedImage.class );
	optionSet.add( BufferedImage[].class );
	optionSet.add( BufferedImageAccessor.class );
	optionSet.add( BufferedImageAccessor[].class );
	optionSet.add( Byte.class );
	optionSet.add( Byte[].class );
	optionSet.add( DataBuffer.class );
	optionSet.add( DataBuffer[].class );
	optionSet.add( Double.class );
	optionSet.add( Double[].class );
	optionSet.add( Float.class );
	optionSet.add( Float[].class );
	optionSet.add( GeneralMetadataTree.class );
	optionSet.add( GeneralMetadataTree[].class );
	optionSet.add( GeneralMetadataValueTree.class );
	optionSet.add( GeneralMetadataValueTree[].class );
	optionSet.add( ImageTree.class );
	optionSet.add( ImageTree[].class );
	optionSet.add( Integer.class );
	optionSet.add( Integer[].class );
	optionSet.add( Long.class );
	optionSet.add( Long[].class );
	optionSet.add( MetadataTree.class );
	optionSet.add( MetadataTree[].class );
	optionSet.add( Object.class );
	optionSet.add( Object[].class );
	optionSet.add( Short.class );
	optionSet.add( Short[].class );
	optionSet.add( SingleSourceDataTree.class );
	optionSet.add( SingleSourceDataTree[].class );
	optionSet.add( String.class );
	optionSet.add( String[].class );

	// Return a Set Restriction
	String name = "Allowed Types";
	String desc = "Types allowed for configurable processors.";
	return new SetRestriction(name, desc, optionSet);
    }

    /**
     * Gets the Restriction for positive numbers.
     *
     * @return Restriction for positive numbers.
     */
    public static Restriction getPositiveRestriction()
    {
	String name = "Non-negative Numbers";
	String desc = "Only numbers greater than or equal to zero are allowed.";
	return new RangeRestriction(name, desc, 0, Double.POSITIVE_INFINITY);
    }
}
