/*
COPYRIGHT NOTICE
Copyright (c) 2003  Scott C. Neu, Daniel J. Valentino, and Arthur W. Toga

See README.license for license notices.
 */

package edu.ucla.loni.debabel.processors.stdlib;

import edu.ucla.loni.flow.InputNode;
import edu.ucla.loni.flow.OutputNode;
import edu.ucla.loni.flow.exceptions.ProcessingException;
import edu.ucla.loni.flow.process.Processor;
import edu.ucla.loni.flow.process.ProcessorProperty;
import java.lang.reflect.Array;

/**
 * Processor that returns the size of an array.
 *
 * @author Scott C. Neu 
 *
 * @version 3 June 2003
 */
public class ArraySizer extends Processor
{
  /** Description of this Processor. */
  private static String DESC = "Computes the size of an array.";

  /** Input Node Name for the array to get the size of. */
  private static String ARRAY_NAME = "Array";

  /** Input Node Description for the array to get the size of. */
  private static String ARRAY_DESC = "Array to get the size of.";

  /** Output Node Name for the array size. */
  private static String SIZE_NAME = "Size";

  /** Output Node Description for the array size. */
  private static String SIZE_DESC = "Size of the array.";

  /**
   * Constructs an Array Sizer with default Properties.
   *
   * @param name Name of the Processor.
   *
   * @throws IllegalStringException If the name contains illegal characters.
   */
  public ArraySizer(String name)
    {
      super(name, DESC);

      // Create the Input Node for the array
      Class type = Object.class;
      InputNode inputNode = new InputNode(ARRAY_NAME, ARRAY_DESC, type);
      _addInputNode(inputNode);

      // Create the Output Node for the array size
      type = Integer.class;
      OutputNode node = new OutputNode(SIZE_NAME, SIZE_DESC, type);
      _addOutputNode(node);
    }

  /**
   * Constructs an Array Sizer using the specified Properties.
   *
   * @param name Name of the Processor.
   * @param properties Properties for the Processor.
   *
   * @throws IllegalStringException If the name contains illegal characters.
   */
  public ArraySizer(String name, ProcessorProperty[] properties)
    {
      this(name);
    }

  /**
   * Generates values for the Output Nodes using the current values of the
   * Input Nodes.
   *
   * @throws ProcessingException If an error occurs during processing.
   */
  public void generateOutputValues() throws ProcessingException
    {
      try {

	// Get the array value
	Object arrayValue = _getInputValue(ARRAY_NAME, null);

	// Get the array size
	Integer size = new Integer(0);
	if ( arrayValue != null && arrayValue.getClass().isArray() ) {
	  size = new Integer( Array.getLength(arrayValue) );
	}

	// Set the array size
	_setOutputValue(SIZE_NAME, size);
      }

      catch (Exception e) {
	String msg = "Unable to generate output values for \"" + this + "\".";
	throw new ProcessingException(msg, e);
      }
    }
}
